import uuid
class Runner:

    def __init__(self,id,start,end,event,event_num,score='F',rbi='F',earned='F'):
        self.uuid = uuid.uuid4()
        self.id = id
        self.start = start
        self.end = end
        self.event = event
        self.event_num = event_num
        self.score = score
        self.rbi = rbi
        self.earned = earned

        self.at_bat_id = None
        self.half_inning_id = None
        self.inning_id = None
        self.game_id = None



class GamePlayer:
    # need to add a game_id to this 
    def __init__(self,_id,first,last,num,boxname,
    rl,bats,position,status,team_abbrev,
    team_id,parent_team_abbrev,parent_team_id,avg,
    hr,rbi,current_position,bat_order,
    game_position,wins,losses,era):
        self.id = _id
        self.first=first
        self.last = last
        self.num=num
        self.boxname = boxname
        self.rl = rl
        self.bats = bats
        self.position = position
        self.status = status
        self.team_abbrev = team_abbrev
        self.team_id = team_id
        self.parent_team_abbrev = parent_team_abbrev
        self.parent_team_id = parent_team_id
        self.avg = avg
        self.hr = hr
        self.rbi = rbi
        self.current_position = current_position
        self.bat_order = bat_order
        self.game_position = game_position
        self.wins = wins
        self.losses = losses
        self.era = era
        self.gid = None
        self.url = None





import uuid
class Action:

    def __init__(self,b,s,o,des,event,tfs,tfs_zulu,player,pitch,event_num,home_team_runs,away_team_runs):
        self.uuid = uuid.uuid4()
        self.b = b
        self.s = s
        self.o = o
        self.des = des
        self.event = event
        self.tfs = tfs
        self.tfs_zulu = tfs_zulu
        self.player = player
        self.pitch = pitch
        self.event_num = event_num
        self.home_team_runs = home_team_runs
        self.away_team_runs = away_team_runs

        self.half_inning_id = None
        self.inning_id = None
        self.game_id = None



from sqlalchemy.sql import text
from pypitchfx.load.queries import *

# Helper methods to create necessary tables
def create_game_table(conn):
    conn.execute(CREATE_GAME_TABLE)

def  create_inning_table(conn):
    conn.execute(CREATE_INNING_TABLE)

def create_half_inning_table(conn):
    conn.execute(CREATE_HALF_INNING_TABLE)

def create_at_bat_table(conn):
    conn.execute(CREATE_AT_BAT_TABLE)

def create_pitch_table(conn):
    conn.execute(CREATE_PITCH_TABLE)

def create_runner_table(conn):
    conn.execute(CREATE_RUNNER_TABLE)

def create_pickoff_table(conn):
    conn.execute(CREATE_PICKOFF_TABLE)

def create_action_table(conn):
    conn.execute(CREATE_ACTION_TABLE)

def create_game_player_table(conn):
    conn.execute(CREATE_GAME_PLAYER_TABLE)

def create_player_tables(conn):
    create_game_player_table(conn)

def create_game_tables(conn):
    create_game_table(conn)
    create_inning_table(conn)
    create_half_inning_table(conn)
    create_at_bat_table(conn)
    create_pitch_table(conn)
    create_runner_table(conn)
    create_pickoff_table(conn)
    create_action_table(conn)

def create_tables(conn,table_names):
    if 'game' not in table_names:
        create_game_table(conn)
    if 'game_player' not in table_names:
        create_game_player_table(conn)
    if 'inning' not in table_names:
        create_inning_table(conn)
    if 'half_inning' not in table_names:
        create_half_inning_table(conn)
    if 'at_bat' not in table_names:
        create_at_bat_table(conn)
    if 'runner' not in table_names:
        create_runner_table(conn)
    if 'pitch' not in table_names:
        create_pitch_table(conn)
    if 'action' not in table_names:
        create_action_table(conn)
    if 'pickoff' not in table_names:
        create_pickoff_table(conn)



from pypitchfx.load.queries import *
from pypitchfx.gameday_model.Action import Action
from pypitchfx.gameday_model.AtBat import AtBat
from pypitchfx.gameday_model.Game import Game 
from pypitchfx.gameday_model.GamePlayer import GamePlayer 
from pypitchfx.gameday_model.Inning import Inning 
from pypitchfx.gameday_model.HalfInning import HalfInning 
from pypitchfx.gameday_model.Pickoff import Pickoff 
from pypitchfx.gameday_model.Pitch import Pitch
from pypitchfx.gameday_model.Runner import Runner 
from pypitchfx.utils.utils import *
from sqlalchemy.sql import text
import logging
logger = logging.getLogger("pypitchfx")


def insert_game(conn,game):
    logger.info('Inserting {}\n'.format(game.url))
    game_id = game.uuid
    at_bat = game.atBat
    deck = game.deck
    hole = game.hole
    ind = game.ind 
    innings = game.innings
    innings_ids = get_ids(innings)
    sql = text(INSERT_GAME.format(
        game_id=game_id,
        at_bat = at_bat,
        deck =deck,
        hole=hole,
        ind=ind,
        innings_id=innings_ids,
        url=game.url,
        gid=game.gid)
        .replace("'None'",'None').replace('None','NULL'))
    conn.execute(sql)


def insert_inning(conn,inning):
    inning_id = str(inning.uuid)
    game_id = inning.game_id
    num = int(inning.num)
    away_team = inning.away_team
    home_team = inning.home_team
    nxt = get_boolean_from_TF(inning.next)
    top_id = str(inning.top.uuid)
    bot_id = None
    if inning.bottom is not None:
        bot_id = str(inning.bottom.uuid)
    sql = text(INSERT_INNING.format(
        inning_id=inning_id,
        num=num,
        away_team=away_team,
        home_team=home_team,
        top_inning_id = top_id,
        bottom_inning_id = bot_id,
        game_id=game_id,
        next=nxt)
        .replace("'None'",'None').replace('None','NULL'))
    conn.execute(sql)

def insert_half_inning_helper(conn,hf):
    game_id = hf.game_id
    inning_id = hf.inning_id
    hf_id = str(hf.uuid)
    isTop = hf.isTop
    at_bats_and_actions_ids = get_ids(hf.at_bats_and_actions)
    sql = text(INSERT_HALF_INNING.format(
        half_inning_id=hf_id,
        inning_id=inning_id,
        game_id = game_id,
        isTop = isTop,
        at_bats_actions_id=at_bats_and_actions_ids)
        .replace("'None'",'None').replace('None','NULL'))
    conn.execute(sql)

def insert_pitch(conn,pitch):
    pitch_id = str(pitch.uuid)
    game_id = pitch.game_id
    inning_id = pitch.inning_id
    hf_id = pitch.half_inning_id
    ab_id = pitch.at_bat_id

    des = pitch.des
    des = des.replace("'","''")
    id_ = pitch.id
    type_ = pitch.type
    tfs_zulu = zulu_to_ts(pitch.tfs_zulu)
    x = pitch.x
    y = pitch.y
    event_num = pitch.event_num
    sv_id = pitch.sv_id
    play_guid = pitch.play_guid
    start_speed = pitch.start_speed
    end_speed = pitch.end_speed
    sz_top = pitch.sz_top
    sz_bot = pitch.sz_bot
    pfx_x = pitch.pfx_x
    pfx_z = pitch.pfx_z
    px = pitch.px
    pz = pitch.pz
    x0 = pitch.x0
    y0 = pitch.y0
    z0 = pitch.z0
    vx0 = pitch.vx0
    vy0 = pitch.vy0
    vz0 = pitch.vz0
    ax = pitch.ax
    ay = pitch.ay
    az = pitch.az
    break_y = pitch.break_y
    break_angle = pitch.break_angle
    break_length = pitch.break_length
    pitch_type = pitch.pitch_type
    type_confidence = pitch.type_confidence
    zone = pitch.zone
    nasty = pitch.nasty
    spin_dir = pitch.spin_dir
    spin_rate = pitch.spin_rate
    cc = ''
    mt = ''
    outcome = get_pitch_outcome(pitch)
    sql = text(INSERT_PITCH.format(
        pitch_id = pitch_id,
        des = des,
        id = id_,
        type = type_,
        tfs_zulu = tfs_zulu,
        x = x,
        y = y,
        event_num = event_num,
        sv_id = sv_id,
        play_guid = play_guid,
        start_speed = start_speed,
        end_speed = end_speed,
        sz_top = sz_top,
        sz_bot = sz_bot,
        pfx_x = pfx_x,
        pfx_z = pfx_z,
        px = px,
        pz = pz,
        x0 = x0,
        y0 = y0,
        z0 = z0,
        vx0 = vx0,
        vy0 = vy0,
        vz0 = vz0,
        ax = ax,
        ay = ay,
        az = az,
        break_y = break_y,
        break_angle = break_angle,
        pitch_type=pitch_type,
        break_length = break_length,
        type_confidence = type_confidence,
        zone = zone,
        nasty = nasty,
        spin_dir = spin_dir,
        spin_rate = spin_rate,
        cc = cc,
        mt = mt,
        game_id = game_id,
        inning_id = inning_id,
        half_inning_id = hf_id,
        at_bat_id = ab_id,
        outcome = outcome
    ).replace("'None'",'None').replace('None','NULL'))
    conn.execute(sql)


def insert_runner(conn,runner):
    runner_id = str(runner.uuid)
    game_id = runner.game_id
    inning_id = runner.inning_id
    hf_id = runner.half_inning_id
    ab_id = runner.at_bat_id
    id_ = runner.id
    score = get_boolean_from_TF(runner.score)
    start = runner_base_to_int(runner.start)
    end = runner_base_to_int(runner.end)
    event = runner.event
    if score:
        end = 4
    rbi = get_boolean_from_TF(runner.rbi)
    earned = get_boolean_from_TF(runner.earned)
    sql = text(INSERT_RUNNER.format(
        runner_id=runner_id,
        id=id_,
        start=start,
        end=end,
        event=event,
        score=score,
        rbi=rbi,
        earned=earned,
        game_id = game_id,
        inning_id = inning_id,
        half_inning_id = hf_id,
        at_bat_id = ab_id)
        .replace("'None'",'None').replace('None','NULL'))
    conn.execute(sql)

def insert_pickoff(conn,pickoff):
    po_id =  str(pickoff.uuid)
    ab_id = pickoff.at_bat_id
    hf_id = pickoff.half_inning_id
    inning_id = pickoff.inning_id
    game_id = pickoff.game_id
    des = pickoff.des
    des = des.replace("'","''")
    event_num = pickoff.event_num
    sql = text(INSERT_PICKOFF.format(
        po_id=po_id,
        des=des,
        event_num=event_num,
        game_id = game_id,
        inning_id = inning_id,
        half_inning_id=hf_id,
        at_bat_id = ab_id)
        .replace("'None'",'None').replace('None','NULL'))
    conn.execute(sql)

def insert_at_bat(conn,ab):
    ab_id = str(ab.uuid)
    game_id = ab.game_id
    inning_id = ab.inning_id
    hf_id = ab.half_inning_id

    num = int(ab.num)
    b = int(ab.balls)
    s = int(ab.strikes)
    o = int(ab.outs)
    start = zulu_to_ts(ab.start_tfs_zulu)
    batter = ab.batter
    stands = ab.stand
    b_height = int(ab.b_height)
    pitcher = ab.pitcher
    p_throws = ab.p_throws
    des = ab.des
    des = des.replace("'","''")

    event_num = int(ab.event_num)
    event = ab.event
    home_team_runs = int(ab.home_team_runs)
    away_team_runs = int(ab.away_team_runs)
    pitch_ids = get_ids(ab.pitches)
    runner_ids = get_ids(ab.runners)
    score = get_boolean_from_TF(ab.score)
    outcome = get_at_bat_outcome(ab)
    sql = text(INSERT_AT_BAT.format(
        at_bat_id=ab_id,
        num=num,b=b,
        s=s,
        o=0,
        start_tfs_zulu=start,
        batter=batter,
        b_height=b_height,
        pitcher=pitcher,
        p_throws=p_throws,
        des=des,
        event_num=event_num,
        event=event,
        home_team_runs=home_team_runs,
        away_team_runs=away_team_runs,
        score=score,
        pitch_ids=pitch_ids,
        runner_ids=runner_ids,
        half_inning_id=hf_id,
        game_id = game_id,
        inning_id = inning_id,
        outcome=outcome,
        stands=stands)
        .replace("'None'",'None').replace('None','NULL'))
    conn.execute(sql)
    for pitch in ab.pitches:
        if isinstance(pitch,Pitch):
            insert_pitch(conn,pitch)
        elif isinstance(pitch,Pickoff):
            insert_pickoff(conn,pitch)
        else:
            raise("Invalid pitch "+ pitch)
    for runner in ab.runners:
        insert_runner(conn,runner)

def insert_action(conn,action):
    game_id = action.game_id
    inning_id = action.inning_id
    hf_id = action.half_inning_id

    a_id = str(action.uuid)
    b = action.b
    s = action.s
    o = action.o
    des = action.des
    des = des.replace("'","''")
    event = action.event
    tfs_zulu= zulu_to_ts(action.tfs_zulu)
    player = action.player
    pitch = action.pitch
    event_num = action.event_num
    home_team_runs = action.home_team_runs
    away_team_runs = action.away_team_runs
    sql = text(INSERT_ACTION.format(
        action_id=a_id,
        b=b,
        s=s,
        o=o,
        des=des,
        event=event,
        tfs_zulu=tfs_zulu,
        player=player,
        pitch=pitch,
        event_num=event_num,
        home_team_runs=home_team_runs,
        away_team_runs=away_team_runs,
        half_inning_id=hf_id,
        game_id = game_id,
        inning_id = inning_id)
        .replace("'None'",'None').replace('None','NULL'))
    conn.execute(sql)

def insert_half_inning(conn,hf,i):
    insert_half_inning_helper(conn,hf)
    game_id = hf.game_id
    inning_id = hf.inning_id
    hf_id = str(hf.uuid)
    for a in hf.at_bats_and_actions:
        if isinstance(a,AtBat):
            insert_at_bat(conn,a)
        elif isinstance(a,Action):
            insert_action(conn,a)

def insert_innings(conn,game):
    innings = game.innings
    for i in innings:
        insert_inning(conn,i)
        insert_half_inning(conn,i.top,i)
        if(i.bottom is not None):
            insert_half_inning(conn,i.bottom,i)

# Load the Game object into a sql DB
def load_game(game,conn):
    insert_game(conn,game)
    insert_innings(conn,game)


import uuid
class Pitch:

    def __init__(self,des,id_,type_,tfs,tfs_zulu,x,y,event_num,sv_id,
    play_guid,start_speed,end_speed,sz_top,sz_bot,pfx_x,pfx_z,
    px,pz,x0,y0,z0,vx0,vy0,vz0,ax,ay,az,break_y,break_angle,break_length,
    pitch_type,type_confidence,zone,nasty,spin_dir,spin_rate,cc,mt):
        self.uuid = uuid.uuid4()
        self.des = des
        self.id = id_
        self.type = type_
        self.tfs = tfs
        self.tfs_zulu = tfs_zulu
        self.x = x
        self.y = y
        self.event_num = event_num
        self.sv_id = sv_id
        self.play_guid = play_guid 
        self.start_speed = start_speed
        self.end_speed = end_speed
        self.sz_top = sz_top
        self.sz_bot = sz_bot
        self.pfx_x = pfx_x
        self.pfx_z = pfx_z
        self.px = px
        self.pz = pz
        self.x0 = x0
        self.y0 = y0
        self.z0 = z0
        self.vx0 = vx0
        self.vy0 = vy0
        self.vz0 = vz0
        self.ax = ax
        self.ay = ay
        self.az = az
        self.break_y = break_y
        self.break_angle = break_angle
        self.break_length = break_length
        self.pitch_type = pitch_type
        self.type_confidence = type_confidence
        self.zone = zone
        self.nasty = nasty
        self.spin_dir = spin_dir
        self.spin_rate = spin_rate
        self.cc = cc
        self.mt = mt

        self.at_bat_id = None
        self.half_inning_id = None
        self.inning_id = None
        self.game_id = None





import uuid
class HalfInning:
    
    def __init__(self):
        self.uuid = uuid.uuid4()
        self.at_bats_and_actions = None
        self.isTop = None

        self.inning_id = None
        self.game_id = None
        


import pypitchfx
from pypitchfx.gameday_model.Action import Action
from pypitchfx.gameday_model.AtBat import AtBat
from pypitchfx.gameday_model.Game import Game 
from pypitchfx.gameday_model.GamePlayer import GamePlayer 
from pypitchfx.gameday_model.HalfInning import HalfInning 
from pypitchfx.gameday_model.Inning import Inning 
from pypitchfx.gameday_model.Pickoff import Pickoff 
from pypitchfx.gameday_model.Pitch import Pitch
from pypitchfx.gameday_model.Runner import Runner 
from pypitchfx.parse.parse import parse_players,parse_innings_all 
from pypitchfx.utils.utils import makeUrls,get_innings_all_urls,get_players_urls,get_args
from pypitchfx.load.create_tables import create_tables
import logging
import time

localtime = time.asctime( time.localtime(time.time()) )

logger = logging.getLogger("pypitchfx")
logger.setLevel(logging.DEBUG)

# create file handler which logs debug messages
fh = logging.FileHandler("scrape_{}.log".format(localtime), 'w', 'utf-8')
fh.setLevel(logging.DEBUG)

# create console handler with an info log level
ch = logging.StreamHandler()
ch.setLevel(logging.INFO)

# creating a formatter
formatter = logging.Formatter('- %(name)s - %(levelname)-8s: %(message)s')

# setting handler format
fh.setFormatter(formatter)
ch.setFormatter(formatter)

# add the handlers to the logger
logger.addHandler(fh)
logger.addHandler(ch)

'''
Python tool to scrape pitchf/x data from MLB's Gameday repo
Inspired by pitchfx package written in R
Author: Javier Palomares
'''

# Return the games and players for the given date range or list of ids
# Pass in a SqlAlchemy engine to write the games and players to a relational DB
def scrape_games_players(start=None,end=None,batch_size=20,game_ids=None,engine=None):
    if game_ids is None:
        if start is None or end is None:
            raise Exception('Specify the start and end dates, or give the game ids')
        else:
            logger.info("Parsing innings all and players starting on {} to {}".format(start,end))
    else:
        logger.info("Parsing innings all and players for game ids " + game_ids)
    game_urls = makeUrls(start,end,game_ids)
    for url in game_urls:
        logger.info(url)
        print(url)
    db_connection = None

    if engine is not None:
        table_names = engine.table_names()
        db_connection = engine.connect()
        create_tables(db_connection,table_names)

    innings_all_urls = get_innings_all_urls(game_urls)
    players_urls = get_players_urls(game_urls)
    num_innings_all_urls = len(innings_all_urls)
    num_player_urls = len(players_urls)
    if num_innings_all_urls != num_player_urls:
        logger.error("There is a mismatch between the number of player and innings_all xml. Bailing out")
        exit(1)
    logger.info("Starting to parse {} innings_all and player xml files from MLB gameday repository in batches of {}.".format(num_innings_all_urls,batch_size))
    # upload the player and game data in batches if the db_connection is not null
    num_to_load = num_innings_all_urls
    index = 0
    games = []
    players = []
    while True:
        start_index = index*batch_size
        end_index = min((index+1)*batch_size,num_to_load)
        logger.info("Loading {} to {} out of {}".format(start_index,end_index,num_to_load))
        if (start_index >= num_to_load):
            break
        index = index + 1
        innings_to_load = innings_all_urls[start_index:end_index]
        players_to_load = players_urls[start_index:end_index]
        games.append(parse_innings_all(innings_to_load,db_connection))
        players.append(parse_players(players_to_load,db_connection))
    if db_connection is not None:
        db_connection.close()

    logger.info("parsed {} game and {} players".format(len(games),len(players)))

    return games,players

def main():
    args = get_args()
    start = args.start
    end = args.end
    gids = args.gameId
    batch_size = args.batchSize
    games,players = scrape_games_players(start,end,batch_size,game_ids=gids)

if __name__=="__main__":
    main()



import uuid
class Inning:

    def __init__(self, num, away_team, home_team,nxt):
        self.uuid = uuid.uuid4()
        self.num = num
        self.away_team = away_team
        self.home_team = home_team
        self.next = nxt
        # top and bottom half innings
        self.top = None
        self.bottom = None
        self.game_id = None



import uuid
class AtBat:

    def __init__(self, num, b, s,o, start_tfs, 
        start_tfs_zulu, batter,stand,b_height,pitcher,
        p_throws,des,event_num,event,home_team_runs,away_team_runs,score='F'):
        self.uuid = uuid.uuid4()
        self.num = num
        self.balls = b
        self.strikes = s
        self.outs = o
        self.start_tfs = start_tfs
        self.start_tfs_zulu = start_tfs_zulu
        self.batter = batter
        self.stand = stand
        self.b_height = b_height 
        self.pitcher = pitcher
        self.p_throws = p_throws
        self.des = des
        self.event_num = event_num
        self.event = event
        self.home_team_runs = home_team_runs
        self.score=score
        self.away_team_runs = away_team_runs
        self.pitches = None
        self.runners = None

        self.half_inning_id = None
        self.inning_id = None
        self.game_id = None


from pypitchfx.load.queries import *
from pypitchfx.gameday_model import GamePlayer
from pypitchfx.utils.utils import *
from sqlalchemy.sql import text

import logging
logger = logging.getLogger("pypitchfx")

def insert_player(conn,player):
    logger.info('Inserting {}\n'.format(player.url))
    player_id = player.id
    first = player.first
    last = player.last
    boxname = player.boxname
    num = player.num
    rl = player.rl
    bats = player.bats
    position = player.position
    _status = player.status
    team_abbrev = player.team_abbrev
    team_id = player.team_id
    parent_team_abbrev = player.parent_team_abbrev
    parent_team_id = player.parent_team_id
    avg = player.avg
    hr = player.hr
    rbi = player.rbi
    current_position = player.current_position
    bat_order = player.bat_order
    game_position = player.game_position
    wins = player.wins
    losses = player.losses
    era = player.era
    gid = player.gid
    bat_order = player.bat_order
    game_position = player.game_position

    # Escape names with single quote (like O'Connor)
    first = first.replace("'","''")
    last = last.replace("'","''")
    boxname = boxname.replace("'","''")
    if not num:
        num = 0
    # some players have '-' to indicate NULL era
    if era is not None and '-' in era:
        era = None
        
    sql = text(INSERT_GAMEPLAYER.format(
        id=player_id,
        first=first,
        last=last,
        num=num,
        boxname=boxname,
        rl=rl,
        bats=bats,
        position=position,
        status=_status,
        team_abbrev=team_abbrev,
        team_id=team_id,
        parent_team_abbrev=parent_team_abbrev,
        parent_team_id=parent_team_id,
        avg=avg,
        hr = hr,
        rbi=rbi,
        current_position=current_position,
        bat_order=bat_order,
        wins=wins,
        losses=losses,
        era=era,
        game_position=game_position,
        gid=gid)
        .replace("'None'",'None').replace('None','NULL'))
    conn.execute(sql)

def load_player(player,conn):
    insert_player(conn,player)



import uuid
class Game:

    def __init__(self, atBat, deck, hole,ind):
        self.uuid = uuid.uuid4()
        self.atBat = atBat
        self.deck = deck
        self.hole = hole
        self.ind = ind
        self.innings = None
        self.url = None
        self.gid = None
    



import uuid
class Pickoff:

    def __init__(self, des,event_num):
        self.uuid = uuid.uuid4()
        self.des = des
        self.event_num = event_num

        self.at_bat_id = None
        self.half_inning_id = None
        self.inning_id = None
        self.game_id = None



from setuptools import setup

setup(name='pypitchfx',
      version='1.0.4',
      description='Tool for parsing mlb gameday data',
      url='http://github.com/JavierPalomares90/pypitchfx',
      author='Javier Palomares',
      author_email='javier.palomares.90@gmail.com',
      license='MIT',
      packages=['pypitchfx','pypitchfx.gameday_model','pypitchfx.load'],
      install_requires=[
          'beautifulsoup4',
          'lxml',
	  'sqlalchemy'
      ],
      zip_safe=False)



'''
SQL statements for creating
and populating the tables
'''

INSERT_PLAYERS = """
INSERT INTO player(
    player_id,
    full_name) 
VALUES(
    '{player_id}',
    '{full_name}');
"""

INSERT_GAME = """
INSERT INTO game(
    game_id,
    atbat,
    deck,
    hole,
    ind,
    innings_ids,
    url,
    gid) 
VALUES(
    '{game_id}',
    '{at_bat}',
    '{deck}',
    '{hole}',
    '{ind}',
    ARRAY{innings_id},
    '{url}',
    '{gid}'
);
"""

INSERT_INNING = """
INSERT INTO inning(
    inning_id,num,
    away_team,
    home_team,
    next,
    top_inning_id,
    bottom_inning_id,
    game_id) 
VALUES(
    '{inning_id}',
    {num},
    '{away_team}',
    '{home_team}',
    {next},
    '{top_inning_id}',
    '{bottom_inning_id}',
    '{game_id}'
);"""

INSERT_HALF_INNING = """
INSERT INTO half_inning(
    half_inning_id,
    at_bats_actions_ids,
    isTop,
    game_id,
    inning_id)
VALUES(
    '{half_inning_id}',
    ARRAY{at_bats_actions_id}::VARCHAR(36)[],
    {isTop},
    '{game_id}',
    '{inning_id}'
);"""

INSERT_ACTION="""
INSERT INTO action(
    action_id,
    b,
    s,
    o,
    des,
    event,
    tfs_zulu,
    player,
    pitch,
    event_num,
    home_team_runs,
    away_team_runs,
    game_id,
    inning_id,
    half_inning_id)
VALUES(
    '{action_id}',
    {b},
    {s},
    {o},
    '{des}',
    '{event}',
    '{tfs_zulu}',
    '{player}',
    {pitch},
    {event_num},
    {home_team_runs},
    {away_team_runs},
    '{game_id}',
    '{inning_id}',
    '{half_inning_id}'
);"""

INSERT_AT_BAT="""
INSERT INTO at_bat(
    at_bat_id,
    num,
    b,
    s,
    o,
    start_tfs_zulu,
    batter,
    stand,
    b_height,
    pitcher,
    p_throws,
    des,
    event_num,
    event,
    home_team_runs,
    away_team_runs,
    score,
    pitch_ids,
    runner_ids,
    game_id,
    inning_id,
    half_inning_id,
    outcome)
VALUES(
    '{at_bat_id}',
    {num},
    {b},
    {s},
    {o},
    '{start_tfs_zulu}',
    '{batter}',
    '{stands}',
    {b_height},
    '{pitcher}',
    '{p_throws}',
    '{des}',
    {event_num},
    '{event}',
    {home_team_runs},
    {away_team_runs},
    {score},
    ARRAY{pitch_ids},
    ARRAY{runner_ids}::VARCHAR(36)[],
    '{game_id}',
    '{inning_id}',
    '{half_inning_id}',
    {outcome}
);"""

INSERT_PITCH="""INSERT INTO pitch(
    pitch_id,
    des,
    id,
    type,
    tfs_zulu,
    x,
    y,
    event_num,
    sv_id,
    play_guid,
    start_speed,
    end_speed,
    sz_top,
    sz_bot,
    pfx_x,
    pfx_z,
    px,
    pz,
    x0,
    y0,
    z0,
    vx0,
    vy0,
    vz0,
    ax,
    ay,
    az,
    break_y,
    break_angle,
    break_length,
    pitch_type,
    type_confidence,
    zone,
    nasty,
    spin_dir,
    spin_rate,
    cc,
    mt,
    game_id,
    inning_id,
    half_inning_id,
    at_bat_id,
    outcome)
VALUES(
    '{pitch_id}',
    '{des}',
    {id},
    '{type}',
    '{tfs_zulu}',
    {x},
    {y},
    {event_num},
    '{sv_id}',
    '{play_guid}',
    {start_speed},
    {end_speed},
    {sz_top},
    {sz_bot},
    {pfx_x},
    {pfx_z},
    {px},
    {pz},
    {x0},
    {y0},
    {z0},
    {vx0},
    {vy0},
    {vz0},
    {ax},
    {ay},
    {az},
    {break_y},
    {break_angle},
    {break_length},
    '{pitch_type}',
    {type_confidence},
    {zone},
    {nasty},
    {spin_dir},
    {spin_rate},
    '{cc}',
    '{mt}',
    '{game_id}',
    '{inning_id}',
    '{half_inning_id}',
    '{at_bat_id}',
    {outcome}
);"""

INSERT_RUNNER="""
INSERT INTO runner(
    runner_id,
    id,
    start_base,
    end_base,
    event,
    score,
    rbi,
    earned,
    game_id,
    inning_id,
    half_inning_id,
    at_bat_id)
VALUES(
    '{runner_id}',
    '{id}',
    {start},
    {end},
    '{event}',
    {score},
    {rbi},
    {earned},
    '{game_id}',
    '{inning_id}',
    '{half_inning_id}',
    '{at_bat_id}'
);"""

INSERT_PICKOFF="""
INSERT INTO pickoff(
    po_id,
    des,
    event_num,
    game_id,
    inning_id,
    half_inning_id,
    at_bat_id)
VALUES (
    '{po_id}',
    '{des}',
    {event_num},
    '{game_id}',
    '{inning_id}',
    '{half_inning_id}',
    '{at_bat_id}'
);"""

INSERT_GAMEPLAYER="""
INSERT INTO game_player(
    id,
    first,
    last,
    num,
    boxname,
    rl,
    bats,
    position,
    status,
    team_abbrev,
    team_id,
    parent_team_abbrev,
    parent_team_id,
    avg,
    hr,
    rbi,
    current_position,
    bat_order,
    game_position,
    wins,
    losses,
    era,
    gid)
VALUES(
    '{id}',
    '{first}',
    '{last}',
    {num},
    '{boxname}',
    '{rl}',
    '{bats}',
    '{position}',
    '{status}',
    '{team_abbrev}',
    '{team_id}',
    '{parent_team_abbrev}',
    '{parent_team_id}',
    {avg},
    {hr},
    {rbi},
    '{current_position}',
    {bat_order},
    '{game_position}',
    {wins},
    {losses},
    {era},
    '{gid}'
);
"""

CREATE_GAME_TABLE="""
CREATE TABLE game(
    game_id VARCHAR(36) PRIMARY KEY,
    atBat VARCHAR (6)  NOT NULL,
    deck VARCHAR (6) NOT NULL,
    hole VARCHAR (6),
    ind TEXT,
    innings_ids VARCHAR(36)[] NOT NULL,
    url TEXT,
    gid TEXT UNIQUE);
"""

CREATE_INNING_TABLE="""
CREATE TABLE inning(
    inning_id VARCHAR(36) PRIMARY KEY,
    num INTEGER NOT NULL,
    away_team VARCHAR(50) NOT NULL,
    home_team VARCHAR(5) NOT NULL,
    next BOOLEAN NOT NULL,
    top_inning_id VARCHAR(36) NOT NULL,
    bottom_inning_id VARCHAR(36),
    game_id VARCHAR(36) NOT NULL,
    FOREIGN KEY (game_id) REFERENCES game(game_id) ON DELETE CASCADE);
"""

CREATE_HALF_INNING_TABLE="""
CREATE TABLE half_inning(
    half_inning_id VARCHAR(36) PRIMARY KEY,
    at_bats_actions_ids VARCHAR(36)[] NOT NULL,
    isTop BOOLEAN NOT NULL,
    game_id VARCHAR(36) NOT NULL,
    inning_id VARCHAR(36) NOT NULL,
    FOREIGN KEY (inning_id) REFERENCES inning(inning_id) ON DELETE CASCADE);
"""

CREATE_AT_BAT_TABLE="""
CREATE TABLE at_bat(
    at_bat_id VARCHAR(36) PRIMARY KEY,
    num INTEGER NOT NULL,
    b INTEGER NOT NULL,
    o INTEGER NOT NULL,
    start_tfs_zulu TIMESTAMP,
    batter VARCHAR(6) NOT NULL,
    stand VARCHAR(1),
    b_height INTEGER,
    pitcher VARCHAR(6) NOT NULL,
    p_throws VARCHAR(1),
    des TEXT,
    event_num INTEGER NOT NULL,
    event TEXT,
    home_team_runs INTEGER NOT NULL,
    away_team_runs INTEGER NOT NULL,
    score BOOLEAN,
    pitch_ids VARCHAR(36)[] NOT NULL,
    runner_ids VARCHAR(36)[],
    outcome NUMERIC NOT NULL,
    s INTEGER,
    game_id VARCHAR(36) NOT NULL,
    inning_id VARCHAR(36) NOT NULL,
    half_inning_id VARCHAR(36),
    FOREIGN KEY (half_inning_id) REFERENCES half_inning(half_inning_id) ON DELETE CASCADE
    );
"""

CREATE_PITCH_TABLE="""
CREATE TABLE pitch( 
    pitch_id VARCHAR(36) PRIMARY KEY,
    des TEXT, 
    id INTEGER,
    type VARCHAR(10), 
    tfs_zulu TIMESTAMP,
    x NUMERIC, 
    y NUMERIC,
    event_num INTEGER, 
    sv_id VARCHAR(13),
    play_guid VARCHAR(50), 
    start_speed NUMERIC,
    end_speed NUMERIC, 
    sz_top NUMERIC,
    sz_bot NUMERIC, 
    pfx_x NUMERIC,
    pfx_z NUMERIC, 
    px NUMERIC,
    pz NUMERIC, 
    x0 NUMERIC,
    y0 NUMERIC, 
    z0 NUMERIC,
    vx0 NUMERIC, 
    vy0 NUMERIC,
    vz0 NUMERIC, 
    ax NUMERIC,
    ay NUMERIC, 
    az NUMERIC,
    break_y NUMERIC, 
    break_angle NUMERIC,
    break_length NUMERIC, 
    pitch_type VARCHAR(10),
    type_confidence NUMERIC, 
    zone INTEGER,
    nasty INTEGER, 
    spin_dir NUMERIC,
    spin_rate NUMERIC, 
    cc  VARCHAR(10),
    mt  VARCHAR(10), 
    outcome NUMERIC,
    game_id VARCHAR(36) NOT NULL,
    inning_id VARCHAR(36) NOT NULL, 
    half_inning_id VARCHAR(36),
    at_bat_id VARCHAR(36) NOT NULL,
    FOREIGN KEY (at_bat_id) REFERENCES at_bat(at_bat_id) ON DELETE CASCADE);
"""

CREATE_RUNNER_TABLE="""
CREATE TABLE runner( 
    runner_id VARCHAR(36) PRIMARY KEY, 
    id VARCHAR(6) NOT NULL, 
    start_base INTEGER NOT NULL, 
    end_base INTEGER, 
    event TEXT, 
    score BOOLEAN, 
    rbi BOOLEAN, 
    earned BOOLEAN, 
    game_id VARCHAR(36) NOT NULL,
    inning_id VARCHAR(36) NOT NULL, 
    half_inning_id VARCHAR(36),
    at_bat_id VARCHAR(36) NOT NULL,
    FOREIGN KEY (at_bat_id) REFERENCES at_bat(at_bat_id) ON DELETE CASCADE);
"""

CREATE_PICKOFF_TABLE="""
CREATE TABLE pickoff(
    po_id VARCHAR(36) PRIMARY KEY,
    des TEXT,
    event_num INTEGER,
    game_id VARCHAR(36) NOT NULL,
    inning_id VARCHAR(36) NOT NULL, 
    half_inning_id VARCHAR(36),
    at_bat_id VARCHAR(36) NOT NULL,
    FOREIGN KEY (at_bat_id) REFERENCES at_bat(at_bat_id) ON DELETE CASCADE);
"""

CREATE_ACTION_TABLE="""
CREATE TABLE action( 
    action_id VARCHAR(36) PRIMARY KEY, 
    b INTEGER, 
    s INTEGER, 
    o INTEGER, 
    des TEXT, 
    event TEXT, 
    tfs_zulu TIMESTAMP, 
    player VARCHAR(6), 
    pitch INTEGER, 
    event_num INTEGER, 
    home_team_runs INTEGER, 
    away_team_runs INTEGER, 
    game_id VARCHAR(36) NOT NULL,
    inning_id VARCHAR(36) NOT NULL, 
    half_inning_id VARCHAR(36),
    FOREIGN KEY (half_inning_id) REFERENCES half_inning(half_inning_id) ON DELETE CASCADE);
"""

CREATE_GAME_PLAYER_TABLE="""
CREATE TABLE game_player( 
    id VARCHAR(36), 
    first TEXT, 
    last TEXT, 
    num INTEGER, 
    boxname TEXT, 
    rl TEXT, 
    bats TEXT, 
    position TEXT, 
    status TEXT, 
    team_abbrev TEXT, 
    team_id TEXT, 
    parent_team_abbrev TEXT, 
    parent_team_id TEXT, 
    avg NUMERIC, 
    hr INTEGER, 
    rbi INTEGER, 
    current_position TEXT, 
    bat_order INTEGER, 
    game_position TEXT, 
    wins INTEGER, 
    losses INTEGER, 
    era NUMERIC, 
    gid TEXT,
    PRIMARY KEY(id,gid),
    FOREIGN KEY (gid) REFERENCES game(gid) ON DELETE CASCADE
); 
"""

